<?php

/* smtp.php: smtp routines
*
    Copyright (C) 2002-2005  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id: smtp.php,v 1.147 2006/01/10 22:44:03 sailfrog Exp $
*/

/**
 *  Send a message unaltered to a new recipient
 *
 *  @param      $to             string      to address
 *  @param      $mailbox        string      IMAP folder
 *  @param      $uid            int         message id
 *  @param      $imap_stream    resource    connection to IMAP
 *  @param      $config         array       the configuration array
 *  @param      $config_file    string      location of configuration file
 *  @param      $from           string      from address string
 *  @return                     void
 */
function hm_send_redirect($to, $mailbox, $uid, $imap_stream, $config, $config_file,
                          $from) {
    $str = $_SESSION['str'];
    if (!isset($config['smtp_helo_name']) || $config['smtp_helo_name'] == 'false') {
        $hostname = preg_replace("/^http(s){0,1}:\/\//", '', $config['hostname']);
        $hostname = preg_replace("/:\d{1,}$/", '', $hostname);
    }
    else {
        $hostname = $config['smtp_helo_name'];
    }
    $server = $config['smtp_server'];
    $domain = $config['domain'];
    $lost = $str[179];

    /* connect to the SMTP server */
    if (!$smtp_stream = hm_connect_smtp($server)) {
        return false;
    }
    
    /* say hello */
    if (!hm_smtp_helo($smtp_stream, $hostname, $lost)) {
        if ($smtp_stream) {
            fclose($smtp_stream);
        }
        return false;
    }
    
    /* smtp auth */
    if (isset($config['smtp_auth']) && ($config['smtp_auth'] == 'cram-md5' ||
        $config['smtp_auth'] == 'plain' || $config['smtp_auth'] == 'login')) {
        if(!hm_auth_smtp($smtp_stream, $config['smtp_auth'], $lost, $config,
            $hasty_key)) {
            if ($smtp_stream) {
                fclose($smtp_stream);
            }
            return false;
        }
    }
       
    if (!hm_smtp_mail($smtp_stream, $lost, $from)) {
        if ($smtp_stream) {
            fclose($smtp_stream);
        }
        return false;
    }
    if (!hm_smtp_rcpt($smtp_stream, $to, $lost)) {
        if ($smtp_stream) {
            fclose($smtp_stream);
        }
    }
    $message = hm_get_message_part($imap_stream, $uid, 'raw');
    $message = preg_replace("/\ndelivered-to:.*/i\n", '', $message);
    $message = preg_replace("/\n/", "\r\n", $message);
    if (!hm_smtp_data($smtp_stream, $message, $lost, true, false, false)) {
        if (is_resource($smtp_stream)) {
            fclose($smtp_stream);
        }
        return false;
    }
    else {
        $_SESSION['notices'][] = 'Redirected message';
    }
}

/**
 *  the main function for sending mail, is called from compose.php
 *
 *  @param  $to                 string      to addresses string
 *  @param  $cc                 string      cc addresses string
 *  @param  $bcc                string      bcc addresses string
 *  @param  $subject            string      subject string
 *  @param  $message            string      message string
 *  @param  $from               string      from address string
 *  @param  $reply_inreplyto    string      inreplyto header value for replies
 *  @param  $reply_messid       string      the message id of the message replied to
 *  @param  $reply_refs         string      the references header value for replies
 *  @param  $reply              boolean     true if we are replying to a message
 *  @param  $config             array       the configuration array
 *  @param  $config_file        string      location of configuration file
 *  @param  $bounce             boolean     if the message is a fake message bounce
 *  @param  $version            string      hastymail version number
 *  @param  $hasty_key          string      encryption key
 *  @param  $imap_stream        resource    connection to imap
 *  @param  $no_save            boolean     true if we skip saving to the sent folder
 *  @param  $priority           int         priority level for the message
 *  @param  $mdn                boolean     true to request mdn response
 *  @param  $dsn                boolean     true to request dsn response
 *  @param  $no_user_agent      boolean     true will skip the user agent header
 *  @param  draft               int         uid of a resumed draft
 *  @param  ean                 boolean     edit as new flag
 *  @return                     boolean     false on failure and true on success
 *
 */
function hm_send_mail($to, $cc, $bcc, $subject, $message, $from, $reply_inreplyto,
                      $reply_messid, $reply_refs, $reply, $config, $config_file,
                      $bounce=false, $version, $hasty_key, $imap_stream, $no_save=false,
                      $priority=false, $mdn=false, $dsn=false, $no_user_agent=false,
                      $draft=false, $ean=false, $preformat=false) {

    $str = $_SESSION['str'];
    if (!isset($config['smtp_helo_name']) || $config['smtp_helo_name'] == 'false') {
        $hostname = preg_replace("/^http(s){0,1}:\/\//", '', $config['hostname']);
        $hostname = preg_replace("/:\d{1,}$/", '', $hostname);
    }
    else {
        $hostname = $config['smtp_helo_name'];
    }
    $server = $config['smtp_server'];
    $domain = $config['domain'];
    $no_reply_refs = false;
    if ($reply) {
        if ($reply_messid == '') {
            $no_reply_refs = true;
        }
        else {
            $irt = $reply_messid;
            if ($reply_refs != '') {
                $refs = $reply_refs.' '.$reply_messid;
            }
            else {
                if ($reply_inreplyto != '') {
                    $refs = $reply_inreplyto.' ';
                }
                $refs = $reply_messid;
            }
            $refs = preg_replace("/(\t|\r\n)/", ' ', $refs);
            $refs = preg_replace("/\s{2,}/", ' ', $refs);
            $refs_array = split(' ', $refs); 
            $refs = implode("\r\n            ", $refs_array);
        }
    }
    if (!$bounce) {
        $from_address = $_SESSION['profiles'][$from]['from_address'];
        if (isset($_SESSION['profiles'][$from]['from_name']) &&
            $_SESSION['profiles'][$from]['from_name'] != '') {
            $from_name = '"'.$_SESSION['profiles'][$from]['from_name'].'"';
        }
        else {
            $from_name = '';
        }
    }
    else {
        $from_address = $from;  
        $from_name = '';
    }
    
    $lost = $str[179];

    /* connect to the SMTP server */
    if (!$smtp_stream = hm_connect_smtp($server)) {
        return false;
    }
    
    /* say hello */
    if (!hm_smtp_helo($smtp_stream, $hostname, $lost)) {
        if ($smtp_stream) {
            fclose($smtp_stream);
        }
        return false;
    }
    
    /* smtp auth */
    if (isset($config['smtp_auth']) && ($config['smtp_auth'] == 'cram-md5' ||
        $config['smtp_auth'] == 'plain' || $config['smtp_auth'] == 'login')) {
        if(!hm_auth_smtp($smtp_stream, $config['smtp_auth'], $lost, $config,
            $hasty_key)) {
            if ($smtp_stream) {
                fclose($smtp_stream);
            }
            return false;
        }
    }
       
    if (!hm_smtp_mail($smtp_stream, $lost, $from_address)) {
        if ($smtp_stream) {
            fclose($smtp_stream);
        }
        return false;
    }
    
    /* parse To:, Cc:, and Bcc: addresses */
    if (isset($config['idna_convert']) && $config['idna_convert'] == 'true') {
        $idna = true;
        require_once('../idna/idna.php');
        if (substr(PHP_VERSION, 0, 1) == 5) {
            require_once('../idna/idna_convert.class.php5.php');
        }
        else {
            require_once('../idna/idna_convert.class.php');
        }    
        $IDN = new Net_IDNA();
    }
    else {
        $idna = false;
    }
    $recpts = array('to' => $to, 'cc' => $cc, 'bcc' => $bcc);
    $no_valid = 0;
    foreach ($recpts as $key => $val) { 
        if ($val != '') {
            $val_array = hm_parse_address($val);
            if (!empty($val_array)) {
                if ($idna) {
                    $val_array = hm_idna_convert($val_array, $IDN);
                }
                $$key = str_replace(array('<', '>'), '', $$key);
                foreach ($val_array as $val_addy) {
                    $$key = str_replace($val_addy, '<'.$val_addy.'>', $$key);
                    if (!hm_smtp_rcpt($smtp_stream, $val_addy, $lost)) {
                        $_SESSION['notices'][] = $str[303].$val_addy;
                    }
                }
            }
            else {
                $no_valid++;
            }
        }
        else {
            $no_valid++;
        }
    }
    if ($no_valid == 3) {
        $_SESSION['notices'][] = $str[344];
        return false;
    }
    /* check fields for required encodings */
    if (hm_special_char_check($from_name)) {
        $from_name = '=?'.$str['charset'].'?B?'.base64_encode($from_name).'?=';
    }
    if (hm_special_char_check($subject)) {
        $subject = '=?'.$str['charset'].'?B?'.base64_encode($subject).'?=';
    }
    $recpts = array('to' => $to, 'cc' => $cc);
    foreach ($recpts as $key => $val) {
        $val = str_replace('"', ' " ', $val);
        $val = preg_replace("/\s{2,}/", ' ', $val);
        $val_parts = explode(' ', $val);
        if (count($val_parts > 1)) {
            $$key = '';
            foreach ($val_parts as $part) {
                if (hm_special_char_check($part)) {
                    $$key .=  ' =?'.$str['charset'].'?B?'.base64_encode($part).'?=';
                }
                else {
                    $$key .= ' '.$part;
                }
            }
        }
        else {
            if (hm_special_char_check($val)) {
                $$key = '=?'.$str['charset'].'?B?'.base64_encode($val).'?=';
            }
        }
        $$key = ltrim($$key);
    }
    if (hm_special_char_check($message)) {
        $content_charset = $str['charset'];
    }
    else {
        $content_charset = 'us-ascii';
    }

    /* build message headers and body to send to the SMTP server */
    $smtp_message  = '';
    if (isset($config['ip_header']) && $config['ip_header'] == 'true') {
        $ip = false;
        if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        }
        elseif (isset($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        }
        else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        if ($ip) {
            $serv = $_SERVER['SERVER_NAME'];
            $smtp_message .= "Received: from [$ip] by $serv with HTTP; ".date("r")."\r\n";
        }
    }
    $smtp_message .= 'Message-ID: <'.md5(uniqid(rand(),1))."@$domain>\r\n";
    $smtp_message .= 'Date: '.date('r', strtotime(date('r')))."\r\n";
    $smtp_message .= "From: $from_name <$from_address>\r\n";
    $smtp_message .= "Subject: $subject\r\n";
    if (trim($to) != '') {
        $smtp_message .= "To: $to\r\n";
    }
    if (trim($cc) != '') {
        $smtp_message .= "Cc: $cc\r\n";
    }
    $smtp_message .= "Reply-to: <$from_address>\r\n";
    if ($reply && !$no_reply_refs) {
        $smtp_message .= "In-Reply-To: ".str_replace(array('&gt;', '&lt;'), 
                                         array('>', '<'), $irt)."\r\n";
        $smtp_message .= "References: ".str_replace(array('&gt;', '&lt;'), 
                                        array('>', '<'), $refs)."\r\n";
    }
    if (!$no_user_agent) {
        if (isset($config['xmailer_header']) && $config['xmailer_header'] != 'none' &&
            $config['xmailer_header'] != 'false' && $config['xmailer_header'] != '') {
            $xmailer = "X-Mailer: ".$config['xmailer_header']."\r\n";
            $smtp_message .= str_replace('$VERSION', $version, $xmailer);
        }
        elseif (!isset($config['xmailer_header']) || (isset($config['xmailer_header']) &&
            ($config['xmailer_header'] == 'false' || $config['xmailer_header'] == ''))) {
            $smtp_message .= "X-Mailer: Hastymail $version\r\n";
        }
    }
    if ($mdn) {
        $smtp_message .= 'disposition-notification-to: '.$from_address."\r\n";
    }
    if ($dsn) {
        $smtp_message .= 'return-receipt-to: '.$from_address."\r\n";
    }
    if ($priority) {
        $smtp_message .= 'x-priority: '.$priority."\r\n";
    }
    $smtp_message .= "MIME-Version: 1.0\r\n";
    $size = 0;
    $bndry = '';
    
    /* calculations and headers for multi-part messages */
    if (isset($_SESSION['attachments']) && count($_SESSION['attachments']) > 0) {
        $bndry = md5(uniqid(rand(),1));
        foreach($_SESSION['attachments'] as $array) {
            $size += ($array['rawsize'] + 6 + strlen($array['header']) + strlen($bndry));
        }
        $size += (strlen($bndry) + 4);
        $smtp_message .= "Content-Type: multipart/mixed; boundary=\"$bndry\"\r\n\r\n";
        $bndry = "--".$bndry;
        $smtp_message .= "$bndry\r\n";
        $smtp_message .= "Content-Type: text/plain; charset=".$content_charset."\r\n";
        if ($preformat) {
            $smtp_message .= "Content-Transfer-Encoding: 8bit\r\n";
        }
        else {
            $smtp_message .= "Content-Transfer-Encoding: 8bit\r\n";
            //$smtp_message .= "Content-Transfer-Encoding: quoted-printable\r\n";
        }
        $multi = true;
        
    }
    else {
        $smtp_message .= "Content-Type: text/plain; charset=".$content_charset."\r\n";
        if ($preformat) {
            $smtp_message .= "Content-Transfer-Encoding: 8bit\r\n";
        }
        else {
            $smtp_message .= "Content-Transfer-Encoding: 8bit\r\n";
            //$smtp_message .= "Content-Transfer-Encoding: quoted-printable\r\n";
        }
        $multi = false;
    }

    /* properly format the body text */
    if (!$preformat) {
        $message = trim($message);
        $lines = split("\n", $message);
        $new_lines = array();
        $new_imap_lines = array();
        foreach($lines as $string) {
            if (strlen($string) > 78) {
                $line = rtrim(implode("\r\n", hm_split_line(rtrim($string), 76, false,
                        true)), "\n")."\r\n";
                /*$line = preg_replace("~([\x01-\x08\x10-\x1F\x3D\x7F-\xFF])~e",
                        "sprintf('=%02X', ord('\\1'))", $line);
                $line = preg_replace("~([\x09\x20])\n~e",
                        "sprintf('=%02X\n', ord('\\1'))", $line);*/
                $new_imap_lines[] = $line;
                $new_lines[] = preg_replace("/^\.\r\n/", "..\r\n", $line);
            }
            else {
                $line = rtrim($string, "\r\n")."\r\n";
                /*$line = preg_replace("~([\x01-\x08\x10-\x1F\x3D\x7F-\xFF])~e",
                        "sprintf('=%02X', ord('\\1'))", $line);
                $line = preg_replace("~([\x09\x20])\n~e",
                        "sprintf('=%02X\n', ord('\\1'))", $line);*/
                $new_imap_lines[] = $line;
                $new_lines[] = preg_replace("/^\.\r\n/", "..\r\n", $line);
            }
        }
    }
    else {
        $lines = split("\n", $message);
        $new_lines = array();
        $new_imap_lines = array();
        foreach($lines as $line) {
            $line = trim($line, "\r\n")."\r\n";
            $new_lines[] = preg_replace("/^\.\r\n/", "..\r\n", $line);
            $new_imap_lines[] = $line;
        }
    }
    $body = implode("", $new_lines);
    $imap_body = implode("", $new_imap_lines);
    $imap_message = $smtp_message."\r\n".$imap_body;
    $smtp_message .= "\r\n".$body;
    $size += strlen($imap_message);
    if (strlen($imap_body) == 2) {
        $imap_message .= "\r\n\r\n";
        $size -= 4;
    }

    /* send the headers/body */
    $no_notice = false;
    if ($reply || $bounce) {
        $no_notice = true;
    }
    if (!hm_smtp_data($smtp_stream, $smtp_message, $lost, $no_notice, $multi, $bndry)) {
        if (is_resource($smtp_stream)) {
            fclose($smtp_stream);
        }
        return false;
    }
    /* clean up */
    else {
        fputs($smtp_stream, "QUIT\r\n");
        fclose($smtp_stream);
        $config = $_SESSION['config'];
        if (!$no_save) {
            hm_save_message($imap_message, $imap_stream, 'sent', $multi, $bndry, $size);
            if (isset($_SESSION['message_count_select'][$_SESSION['settings']
                ['sent_folder']])) {
                $_SESSION['message_count_select'][$_SESSION['settings']['sent_folder']]++;
                $_SESSION['changed'][$_SESSION['settings']['sent_folder']] = 1;
            }
            elseif (isset($_SESSION['message_count'][$_SESSION['settings']
                ['sent_folder']])) {
                $_SESSION['message_count'][$_SESSION['settings'] ['sent_folder']]++;
                $_SESSION['changed'][$_SESSION['settings']['sent_folder']] = 1;
            }
        }
        if ($draft && !$ean && isset($_SESSION['settings']['delete_draft']) &&
            $_SESSION['settings']['delete_draft'] == 1) {
            hm_select_mailbox($imap_stream, $_SESSION['settings']['drafts_folder'], 
                              false, false);
            hm_message_controls('delete', $imap_stream, $draft, false, false, false,
                                $_SESSION['settings']['drafts_folder'], 1, $str[26],
                                false);
        }
        hm_imap_logout($imap_stream); 
        return true;
    }
}

/**
 *  connect to the SMTP server
 *
 *  @param      $server     string      smtp server address or hostname
 *  @return                 resource    false on failure
 *
 */
function hm_connect_smtp($server) {
    $str = $_SESSION['str'];
    if (isset($_SESSION['config']['smtp_port'])) {
        $port = $_SESSION['config']['smtp_port'];
    }
    else {
        $port = 25;
    }
    if (isset($_SESSION['config']['smtp_ssl']) && 
        $_SESSION['config']['smtp_ssl'] == 'ssl') {
        $server = 'ssl://'.$server;
    }
    elseif (isset($_SESSION['config']['smtp_ssl']) && 
        $_SESSION['config']['smtp_ssl'] == 'tls') {
        $server = 'tls://'.$server;
    }
    if ($smtp_stream = @fsockopen($server, $port, $errorno, $errorstr, 30)) {
        if ($helo = fgets($smtp_stream, 1024)) {
            if (substr($helo, 0, 4) == '220 ') {
                return $smtp_stream;
            }
            elseif (substr($helo, 0, 4) == '220-') {
                while (substr($helo, 0, 4) == '220-') {
                    $helo = fgets($smtp_stream, 1024);
                }
                return $smtp_stream;
            }
            else {
                $_SESSION['notices'][] = $str[180].':<br>'.$helo;
                return false;
            }
        }
        else {
            $_SESSION['notices'][] = $str[181];
            return false;
        }
    }
    else {
        $_SESSION['notices'][] = $str[181];
        return false;
    }
}
    
/**
 *  sends the SMTP HELO command
 *
 *  @param      $smtp_stream    resource    connection to the smtp server
 *  @param      $hostname       string      the hostname we are sending mail from
 *  @return                     boolean     true on success, false on failure
 *
 */
function hm_smtp_helo($smtp_stream, $hostname) {
    $str = $_SESSION['str'];
    if (isset($_SESSION['config']['smtp_auth']) && 
        ($_SESSION['config']['smtp_auth'] == 'cram-md5' ||
        $_SESSION['config']['smtp_auth'] == 'plain' || 
        $_SESSION['config']['smtp_auth'] == 'login')) {
        fputs($smtp_stream, "EHLO $hostname\r\n");
    }
    else {
        fputs($smtp_stream, "HELO $hostname\r\n");
    }
    if ($response = fgets($smtp_stream, 1024)) {
        if (substr($response, 0, 4) == '250 ') {
            return true;
        }
        elseif (substr($response, 0, 4) == '250-') {
            while (substr($response, 0, 4) == '250-') {
                $response = fgets($smtp_stream, 1024);
            }
            return true;
        }
        else {
            $_SESSION['notices'][] = $str[182].':<br> '.$response;
            return false;
        }
        
    }
    else {
        $_SESSION['notices'][] = $lost;
        return false;
    }
}

/**
 *  smtp authentication routine
 *
 *  @param  $smtp_stream    resource    connection to smtp server
 *  @param  $auth           string      authentication type
 *  @param  $lost           string      failure to connect message
 *  @param  $config         array       configuration values
 *  @param  $hasty_key      string      encryption key
 *  @return                 boolean     true on success, false on failure
 *
 */
function hm_auth_smtp($smtp_stream, $auth, $lost, $config, $hasty_key) {
    $str = $_SESSION['str'];
    /*if (isset($_SESSION['settings']['alt_smtpname']) &&
        $_SESSION['settings']['alt_smtpname'] != '' && isset($_SESSION['alt_spass'])) {
        $user = $_SESSION['settings']['alt_smtpname'];
        $key = hmw_crypt($_SESSION['alt_spass'], $config, 'decrypt', $hasty_key);
    }*/
    //else {
        $key = hmw_crypt($_SESSION['pass'], $config, 'decrypt', $hasty_key);
        $user = $_SESSION['user'];
    //}
    if ($auth != 'plain') {
        $command = 'AUTH '.strtoupper($auth)."\r\n";
    }
    else {
        $command = 'AUTH PLAIN '.base64_encode($user."\0".$user."\0".$key)."\r\n";
    }
    fputs($smtp_stream, $command);
    if ($response = fgets($smtp_stream, 1024)) {
        if (substr($response, 0, 3) != '334' && $auth != 'plain') {
            $_SESSION['notices'][] = $str[234].':<br>'.$response;
            return false;
        }
        else {
            switch ($auth) {
                case 'cram-md5':
                    $challenge = base64_decode(substr($response, 4));
                    $key .= str_repeat(chr(0x00), (64-strlen($key)));
                    $ipad = str_repeat(chr(0x36), 64);
                    $opad = str_repeat(chr(0x5c), 64);
                    $digest = bin2hex(pack("H*", md5(($key ^ $opad).
                              pack("H*", md5(($key ^ $ipad).$challenge)))));
                    $challenge_response = base64_encode($user.' '.$digest);
                    fputs($smtp_stream, $challenge_response."\r\n");
                    $result = fgets($smtp_stream, 1024);
                    break;
                case 'login':
                    fputs($smtp_stream, base64_encode($user)."\r\n");
                    $prompt = fgets($smtp_stream, 1024);
                    fputs($smtp_stream, base64_encode($key)."\r\n");
                    $result = fgets($smtp_stream, 1024);
                    break;
                case 'plain':
                    $result = $response;
                    break;
                default:
                   $result = $response;
                    break;
            }
            if (substr($result, 0, 3) != '235') {
                $_SESSION['notices'][] = $str[234].':<br> '.$result;
                return false;
            }
            else {
                return true;
            }
        }
    }
    else {
        $_SESSION['notices'][] = $lost;
        return false;
    }
}

/**
 *  performs the SMTP MAIL command
 *
 *  @param  $smtp_stream    resource    connection to the smtp server
 *  @param  $lost           string      failure to connect message
 *  @param  $from           string      the senders email address
 *  @return                 boolean     true on success, false on failure
 *
 */
function hm_smtp_mail ($smtp_stream, $lost, $from) {
    $str = $_SESSION['str'];
    fputs($smtp_stream, "MAIL FROM: <$from>\r\n");
    if ($response = fgets($smtp_stream, 1024)) {
        if (preg_match("/^250/", $response)) {
            return true;
        }
        else {
            $_SESSION['notices'][] = $str[183].':<br> '.$response;
            return false;
        }
    }
    else {
        $_SESSION['notices'][] = $lost;
        return false;
    }
}

/**
 *  performs the SMTP RCPT command
 *
 *  @param      $smtp_stream    resource    connection to the smtp server
 *  @param      $to             string      a single to address
 *  @param      $lost           string      failure to connect message
 *  @return                     boolean     true on success, false on failure
 *
 */
function hm_smtp_rcpt ($smtp_stream, $to, $lost) {
    $str = $_SESSION['str'];
    //$dmpart = strrev(strrev(strstr($to, '@')));
    fputs($smtp_stream, "RCPT TO: <$to>\r\n");
    if ($response = fgets($smtp_stream, 1024)) {
        if (preg_match("/^25/", $response)) {
            return true;
        }
        else {
            $_SESSION['notices'][] = $str[262].':<br>'.$response;
            return false;
        }
        
    }
    else {
        $_SESSION['notices'][] = $lost;
        return false;
    }
}

/**
 *  sends the SMTP data (headers and body)
 *
 *  @param      $smtp_stream    resource    connection to the smtp server
 *  @param      $message        string      the message to send
 *  @param      $lost           string      failure to connect message
 *  @param      $reply          boolean     true if the message is a reply
 *  @param      $multi          boolean     true if the message has attachments
 *  @param      $bndry          string      mime boundary
 *  @return                     boolean     true on success, false on failure
 *
 */
function hm_smtp_data ($smtp_stream, $message, $lost, $reply, $multi, $bndry) {
    $str = $_SESSION['str'];
    fputs($smtp_stream, "DATA\r\n");
    if ($response = fgets($smtp_stream, 1024)) {
        if (preg_match("/^354/", $response)) {
            if ($multi) {
                if(!hm_write_data($smtp_stream, $message, $bndry)) {
                    $_SESSION['notices'][] =  $lost; 
                    return false;
                }
                else {
                    fputs($smtp_stream, "\r\n.\r\n");
                }
            }
            else {
                    fputs($smtp_stream, $message."\r\n.\r\n");
            }
            if ($final_response = fgets($smtp_stream, 1024)) {
                if (preg_match("/^250/", $final_response)) {
                    if (!$reply) {
                        $_SESSION['notices'][] = $str[184];
                    }
                    return true;
                }
                else {
                    $_SESSION['notices'][] = $str[185].':<br>'.$final_response;
                    return false;
                }
            }
            else {
                $_SESSION['notices'][] = $lost; 
                return false;
            }
        }
        else {
            $_SESSION['notices'][] = $str[185].':<br>'.$response;
            return false;
        }
        
    }
    else {
        $_SESSION['notices'][] = $lost;
        return false;
    }
}

/**
 *  checks for non-ascii chars.
 *
 *  @param      $string     string      string to check
 *  @return                 boolean     true if a char over ASCII code 127 is found
 *
 */
function hm_special_char_check($string) {
    $result = false;
    $len = strlen($string);
    for ($i=0;$i<$len;$i++) {
        if (ord($string{$i}) > 127 || ord($string{$i}) == 0) {
            $result = true;
            break;
        }
    }
    return $result;
}

/**
 *  formats attachments for outgoing mail, either from forward, resume_draft
 *  or edit_as_new
 *
 *  @param      $imap_stream    resource    connection to imap 
 *  @param      $structure      array       message structure
 *  @param      $part           string      current message part number
 *  @param      $uid            int         IMAP message uid
 *  @param      $sel            array       only these parts to forward (false for all)
 *  @return                     void
 *
*/
function hm_format_attached ($imap_stream, $structure, $part, $uid, $sel=false) {
    foreach($structure as $part_num => $info_array) {
        if (!$sel || (in_array($part_num, $sel) && $part_num != $part)) {
            if ($sel || ($part_num != $part)) {
                $part = hm_get_message_part ($imap_stream, $uid, $part_num);
                if (preg_match("/[^\r]\n/", $part)) {
                    $part = str_replace("\n", "\r\n", $part);
                }
                $part = rtrim($part)."\r\n";
                if ($structure[$part_num]['type'] == 'message') {
                    $head = hm_get_full_header($imap_stream, $uid, $part_num, true);
                    $part = "\r\n".$head."\r\n\r\n".$part;
                }
                $rawsize = strlen($part);
                if ($structure[$part_num]['rawsize'] == '') {
                    $structure[$part_num]['rawsize'] = $rawsize;
                } 
                $temp_file_name = md5(uniqid(rand(),1));
                if ($rawsize > 0 && $fp = fopen ($_SESSION['config']['settings_dir'].
                    "attachments/$temp_file_name", 'w')) {
                    fputs($fp, $part, strlen($part));
                    fclose($fp);
                    $_SESSION['attachments'][$temp_file_name] = array( 
                            'name'     => hm_decode_field($structure[$part_num]['name']),
                            'type'     => $structure[$part_num]['type'].'/'.
                                            $structure[$part_num]['subtype'],
                            'size'     => $structure[$part_num]['rawsize'],
                            'rawsize'     => $rawsize,
                            'encoding' => $structure[$part_num]['encoding']);
                    if ($structure[$part_num]['type'].'/'.
                        $structure[$part_num]['subtype'] != 'message/digest') {
                        $_SESSION['attachments'][$temp_file_name]['header'] =
                            "Content-Type: ".$structure[$part_num]['type'].
                            '/'.$structure[$part_num]['subtype'].
                            '; name="'.$structure[$part_num]['name']."\"\r\n".
                            'Content-Disposition: attachment; filename="'.
                            $structure[$part_num]['name']."\"\r\n";
                    }
                    if (substr($_SESSION['attachments'][$temp_file_name]['type'], 
                        0, 7) != 'message') {
                        $_SESSION['attachments'][$temp_file_name]['header'] .=
                            'Content-Transfer-Encoding: '.
                            $_SESSION['attachments'][$temp_file_name]['encoding'].
                            "\r\n\r\n";
                    }
                }
            }
        }
        if (isset($structure[$part_num]['subs']) && $sel) {
            hm_format_attached ($imap_stream, $structure[$part_num]['subs'],
                                $part, $uid, $sel);
        }
    }
}
?>
